declare module "lib/utils/MathExtra" {
    import * as math from 'mathjs';
    const _default: math.MathJsStatic & {
        contrib: {
            covariance: (X: any, xMean: any, y: any, yMean: any) => number;
            euclideanDistance: (v1: number[], v2: number[]) => number;
            hstack: (X: any, y: any) => any[];
            isArrayOf: (arr: any, _type?: string) => boolean;
            inner: (a: any, b: any) => any;
            isMatrix: (matrix: any) => boolean;
            isMatrixOf: (matrix: any, _type?: string) => boolean;
            manhattanDistance: (v1: number[], v2: number[]) => number;
            prod: (X: any, axis?: any) => any;
            range: (start: number, stop: number) => number[];
            size: (X: any, axis?: number) => number;
            subtract: (X: any, y: any) => any;
            variance: (X: any, mean: any) => number;
        };
    };
    export default _default;
}
declare module "lib/cluster/k_means" {
    export interface KMeansOptions {
        /**
         * Number of clusters
         */
        k: number;
        /**
         * Choice of distance method. Defaulting to euclidean
         */
        distance?: 'euclidean' | 'manhattan';
        /**
         * Relative tolerance with regards to inertia to declare convergence
         */
        maxIteration?: number;
        /**
         * Random state value for sorting centroids during the getInitialCentroid phase
         */
        randomState?: number;
    }
    /**
     * K-Means clustering
     *
     * @example
     * import { KMeans } from 'kalimdor/cluster';
     *
     * const kmean = new KMeans({ k: 2 });
     * const clusters = kmean.fit({ X: [[1, 2], [1, 4], [1, 0], [4, 2], [4, 4], [4, 0]] });
     *
     * const result = kmean.predict({ X: [[0, 0], [4, 4]] });
     * // results in: [0, 1]
     */
    export class KMeans {
        private assignment;
        private centroids;
        private clusters;
        private distance;
        private k;
        private randomState;
        private maxIteration;
        constructor(options?: KMeansOptions);
        /**
         * Compute k-means clustering.
         * @param {any} X
         * @returns {{centroids: number[]; clusters: number[]}}
         */
        fit({ X }: {
            X: any;
        }): void;
        /**
         * Predicts the cluster index with the given X
         * @param {any} X
         * @returns {number[]}
         */
        predict({ X }: {
            X: any;
        }): number[];
        /**
         * Get the model details in JSON format
         * @returns {{k: number; clusters: number[]; centroids: number[]}}
         */
        toJSON(): {
            k: number;
            clusters: number[];
            centroids: number[];
        };
        /**
         * Restores the model from checkpoints
         * @param {number} k
         * @param {number[]} clusters
         * @param {number[]} centroids
         */
        fromJSON({ k, clusters, centroids }: {
            k: number;
            clusters: number[];
            centroids: number[];
        }): void;
        /**
         * Get initial centroids from X of k
         * @param {number[]} X
         * @param {number} k
         * @returns {number[]}
         */
        private getInitialCentroids;
        /**
         * Get closest centroids based on the passed in distance method
         * @param {number[]} data
         * @param {number[]} centroids
         * @param distance
         * @returns {number}
         */
        private getClosestCentroids;
    }
}
declare module "lib/cluster/index" {
    import { KMeans } from "lib/cluster/k_means";
    export { KMeans };
}
declare module "lib/preprocessing/label" {
    /**
     * Encode labels with value between 0 and n_classes-1.
     *
     * @example
     * import { LabelEncoder } from 'kalimdor/preprocessing';
     *
     * const labelEncoder = new LabelEncoder();
     * const labelX = ['amsterdam', 'paris', 'tokyo'];
     * labelEncoder.fit(labelX);
     * const transformX = ['tokyo', 'tokyo', 'paris'];
     * const result = labelEncoder.transform(transformX);
     * // [ 2, 2, 1 ]
     */
    export class LabelEncoder {
        private classes;
        /**
         * Fit label encoder
         * @param {any[]} X - Input data in array or matrix
         */
        fit(X?: any[]): void;
        /**
         * Transform labels to normalized encoding.
         *
         * Given classes of ['amsterdam', 'paris', 'tokyo']
         *
         * It transforms ["tokyo", "tokyo", "paris"]
         *
         * Into [2, 2, 1]
         * @param X - Input data to transform according to the fitted state
         */
        transform(X?: any[]): any[];
    }
}
declare module "lib/datasets/BaseDataset" {
    import 'isomorphic-fetch';
    /**
     * @ignore
     */
    export class BaseDataset {
        /**
         * fetch load from a multiple
         * @param sources - A list of URLs to fetch the data from
         * @param type - type of data; for example CSV or JSON
         * @param delimiter - specify the data delimiter, which will be used to split the row data
         * @param lastIsTarget - tell the underlying processor that the last index of the dataset is the target data
         * @param trainType - data type to enforce on the training dataset
         * @param targetType - target type to enforce on the target dataset
         * @private
         */
        protected fetchLoad(sources?: any[], { type, delimiter, lastIsTarget, trainType, targetType }?: {
            type?: string;
            delimiter?: string;
            lastIsTarget?: true;
            trainType?: string;
            targetType?: string;
        }): Promise<{
            data: any;
            targets: any;
            labels: any;
        }>;
        /**
         * Load data from the local data folder
         */
        protected fsLoad(type: string, { delimiter, lastIsTarget, trainType, targetType }?: {
            delimiter?: string;
            lastIsTarget?: true;
            trainType?: string;
            targetType?: string;
        }): Promise<{
            data: any;
            targets: any;
            labels: any;
        }>;
        /**
         * Processes CSV type dataset. Returns a training and testing data pair
         * @param data - a raw string data
         * @param delimiter - delimiter to split on
         * @param lastIsTarget - flag to indicate that the last element is the target data
         * @param trainType - training data type to enforce
         * @param targetType - target data type to enforce
         */
        private processCSV;
    }
}
declare module "lib/datasets/Boston" {
    /**
     * References:
     * - https://www.kaggle.com/c/boston-housing/data
     * - https://www.cs.toronto.edu/~delve/data/boston/bostonDetail.html
     */
    import { BaseDataset } from "lib/datasets/BaseDataset";
    /**
     * This dataset contains information collected by the U.S Census Service concerning housing in the area of Boston Mass.
     * It was obtained from the StatLib archive (http://lib.stat.cmu.edu/datasets/boston),
     * and has been used extensively throughout the literature to benchmark algorithms.
     * However, these comparisons were primarily done outside of Delve and are thus somewhat suspect.
     * The dataset is small in size with only 506 cases.
     *
     * The data was originally published by Harrison, D. and Rubinfeld, D.L.
     * `Hedonic prices and the demand for clean air', J. Environ. Economics & Management, vol.5, 81-102, 1978.
     *
     * @example
     * import { Boston } from "kalimdor/datasets";
     *
     * (async function() {
     *   const bostonData = new Boston();
     *   const {
     *     data,
     *     targets,
     *     labels,
     *   } = await bostonData.load();
     * });
     *
     */
    export class Boston extends BaseDataset {
        /**
         * Load the dataset
         */
        load(): Promise<{
            /**
             * Training data
             */
            data: any[][];
            /**
             * Target data
             */
            targets: any[];
            /**
             * Real labels
             */
            labels: string[];
        }>;
    }
}
declare module "lib/datasets/Iris" {
    /**
     * References:
     * - https://archive.ics.uci.edu/ml/datasets/iris
     * - https://en.wikipedia.org/wiki/Iris_flower_data_set
     */
    import 'isomorphic-fetch';
    import { BaseDataset } from "lib/datasets/BaseDataset";
    /**
     * The Iris flower data set or Fisher's Iris data set is a multivariate data set introduced by the British statistician and biologist Ronald Fisher
     * in his 1936 paper The use of multiple measurements in taxonomic problems as an example of linear discriminant analysis.
     *
     * It contains 50 samples with 3 classes of 'Setosa', 'versicolor' and 'virginica'
     *
     * @example
     * import { Iris } from 'kalimdor/datasets';
     *
     * (async function() {
     *   const irisData = new Iris();
     *   const {
     *     data,         // returns the iris data (X)
     *     targets,      // list of target values (y)
     *     labels,       // list of labels
     *     targetNames,  // list of short target labels
     *     description   // dataset description
     *   } = await irisData.load(); // loads the data internally
     * })();
     *
     */
    export class Iris extends BaseDataset {
        private dataSources;
        private descSource;
        /**
         * Load datasets
         */
        load(): Promise<{
            /**
             * Training data
             */
            data: any[][];
            /**
             * Target data
             */
            targets: any[];
            /**
             * Real labels
             */
            labels: string[];
            /**
             * Short labels
             */
            targetNames: string[];
            /**
             * Dataset description
             */
            description: string;
        }>;
    }
}
declare module "lib/datasets/index" {
    import { Boston } from "lib/datasets/Boston";
    import { Iris } from "lib/datasets/Iris";
    export { Boston, Iris };
}
declare module "lib/decomposition/pca" {
    /**
     * Principal component analysis (PCA)
     *
     * Linear dimensionality reduction using Singular Value Decomposition of
     * the data to project it to a lower dimensional space.
     *
     * - It uses the LAPACK implementation of the full SVD
     * - or randomized a randomised truncated SVD by the method of
     * Halko et al. 2009, depending on the shape
     * of the input data and the number of components to extract. (Will be implemented)
     *
     * @example
     * import { PCA } from 'kalimdor/decomposition';
     *
     * const pca = new PCA();
     * const X = [[1, 2], [3, 4], [5, 6]];
     * pca.fit({ X });
     * console.log(pca.components); // result: [ [ 0.7071067811865476, 0.7071067811865474 ], [ 0.7071067811865474, -0.7071067811865476 ] ]
     * console.log(pca.explained_variance); // result: [ [ -0.3535533905932736, 0 ], [ 0, 0.5 ], [ 0.35355339059327373, 0 ] ]
     */
    export class PCA {
        /**
         * Principal axes in feature space, representing the directions of
         * maximum variance in the data. The components are sorted by explained_variance_.
         */
        components: any;
        /**
         * The amount of variance explained by each of the selected components.
         *
         * Equal to n_components largest eigenvalues of the covariance matrix of X.
         */
        explained_variance: any;
        /**
         * Fit the model with X.
         * At the moment it does not take n_components into consideration
         * so it will only calculate Singular value decomposition
         * @param {any} X
         */
        fit({ X }: {
            X: any;
        }): void;
    }
}
declare module "lib/decomposition/index" {
    import { PCA } from "lib/decomposition/pca";
    export { PCA };
}
declare module "lib/tree/tree" {
    /**
     * Question used by decision tree algorithm to determine whether to split branch or not
     * @ignore
     */
    export class Question {
        private features;
        private column;
        private value;
        constructor(features: any, column: any, value: any);
        match(example: any): boolean;
        toString(): string;
    }
    /**
     * According to the given targets array, count occurrences into an object.
     * @param {any[]} targets - list of class: count
     * @returns {}
     * @ignore
     */
    export function classCounts(targets: any[]): {};
    /**
     * A leaf node that classifies data.
     * @ignore
     */
    export class Leaf {
        prediction: any;
        constructor(y: any);
    }
    /**
     * It holds a reference to the question, and to the two children nodes
     * @ignore
     */
    export class DecisionNode {
        question: any;
        trueBranch: any;
        falseBranch: any;
        constructor(question: any, trueBranch: any, falseBranch: any);
    }
    /**
     * A decision tree classifier.
     *
     * @example
     * import { DecisionTreeClassifier } from 'kalimdor/tree';
     * const features = ['color', 'diameter', 'label'];
     * const decision = new DecisionTreeClassifier({ featureLabels: features });
     *
     * const X = [['Green', 3], ['Yellow', 3], ['Red', 1], ['Red', 1], ['Yellow', 3]];
     * const y = ['Apple', 'Apple', 'Grape', 'Grape', 'Lemon'];
     * decision.fit({ X, y });
     * decision.printTree(); // try it out yourself! =)
     *
     * decision.predict({ X: [['Green', 3]] }); // [ 'Apple' ]
     * decision.predict({ X }); // [ [ 'Apple' ], [ 'Apple', 'Lemon' ], [ 'Grape', 'Grape' ], [ 'Grape', 'Grape' ], [ 'Apple', 'Lemon' ] ]
     *
     * @example
     * import { DecisionTreeClassifier } from 'kalimdor/tree';
     * const decision = new DecisionTreeClassifier({ featureLabels: null });
     *
     * const X = [[0, 0], [1, 1]];
     * const Y = [0, 1];
     * decision.fit({ X, y });
     * decision2.predict({ row: [[2, 2]] }); // [ 1 ]
     */
    export interface Options {
        featureLabels?: null | any[];
        verbose?: boolean;
        randomise?: boolean;
    }
    export class DecisionTreeClassifier {
        private featureLabels;
        private tree;
        private verbose;
        private randomise;
        private randomEngine;
        constructor({ featureLabels, verbose, randomise, random_state, tree }?: {
            featureLabels?: any[];
            verbose?: boolean;
            randomise?: boolean;
            random_state?: number;
            tree?: any;
        });
        /**
         * Fit date, which builds a tree
         * @param {any} X
         * @param {any} y
         * @returns {Leaf | DecisionNode}
         */
        fit({ X, y }: {
            X: any[];
            y: any[];
        }): void;
        /**
         * Predict multiple rows
         * @param {any[]} X
         * @returns {any[]}
         */
        predict({ X }: {
            X: any[][];
        }): any;
        /**
         * Returns the model checkpoint
         * @returns {{featureLabels: string[]; tree: any; verbose: boolean; randomise: boolean}}
         */
        toJSON(): {
            featureLabels: string[];
            tree: any;
            verbose: boolean;
            randomise: boolean;
        };
        /**
         * Restores the model from a checkpoint
         * @param {string[]} featureLabels
         * @param {any} tree
         * @param {boolean} verbose
         * @param {boolean} randomise
         */
        fromJSON({ featureLabels, tree, verbose, randomise }: {
            featureLabels: string[];
            tree: any;
            verbose: boolean;
            randomise: boolean;
        }): void;
        /**
         * Recursively print the tree into console
         * @param {string} spacing
         */
        printTree(spacing?: string): void;
        /**
         * Partition X and y into true and false branches
         * @param X
         * @param y
         * @param {Question} question
         * @returns {{trueX: Array<any>; trueY: Array<any>; falseX: Array<any>; falseY: Array<any>}}
         */
        private partition;
        /**
         * Calculate the gini impurity of rows
         * Checkout: https://en.wikipedia.org/wiki/Decision_tree_learning#Gini_impurity
         * @param targets
         * @returns {number}
         */
        private gini;
        /**
         * Information Gain.
         *
         * The uncertainty of the starting node, minus the weighted impurity of
         * two child nodes.
         * @param left
         * @param right
         * @param uncertainty
         * @returns {number}
         */
        private infoGain;
        /**
         * Find the best split for the current X and y.
         * @param X
         * @param y
         * @param {boolean} randomise
         * @returns {{bestGain: number; bestQuestion: any}}
         */
        private findBestSplit;
        /**
         * Interactively build tree until it reaches the terminal nodes
         * @param {any} X
         * @param {any} y
         * @returns {any}
         */
        private buildTree;
        /**
         * Internal predict method separated out for recursion purpose
         * @param {any} row
         * @param {any} node
         * @returns {any}
         * @private
         */
        private _predict;
        /**
         * Private method for printing tree; required for recursion
         * @param {any} node
         * @param {any} spacing
         */
        private _printTree;
    }
}
declare module "lib/ensemble/forest" {
    /**
     * Base RandomForest implementation used by both classifier and regressor
     * @ignore
     */
    export class BaseRandomForest {
        protected trees: any[];
        protected nEstimator: any;
        protected randomState: any;
        /**
         *
         * @param {number} nEstimator - Number of trees.
         * @param random_state - Random seed value for DecisionTrees
         */
        constructor({ nEstimator, random_state }?: {
            nEstimator?: number;
            random_state?: number;
        });
        /**
         * Build a forest of trees from the training set (X, y).
         * @param {Array} X - array-like or sparse matrix of shape = [n_samples, n_features]
         * @param {Array} y - array-like, shape = [n_samples] or [n_samples, n_outputs]
         * @returns void
         */
        fit({ X, y }: {
            X: number[][];
            y: number[];
        }): void;
        /**
         * Returning the current model's checkpoint
         * @returns {{trees: any[]; nEstimator: number}}
         */
        toJSON(): {
            trees: any[];
            nEstimator: number;
        };
        /**
         * Restore the model from a checkpoint
         * @param {any[]} trees - Decision trees
         */
        fromJSON({ trees, nEstimator }: {
            trees: any[];
            nEstimator: number;
        }): void;
        /**
         * Internal predict function used by either RandomForestClassifier or Regressor
         * @param X
         * @private
         */
        protected _predict(X?: number[] | number[][]): any[];
    }
    /**
     * Random forest classifier creates a set of decision trees from randomly selected subset of training set.
     * It then aggregates the votes from different decision trees to decide the final class of the test object.
     *
     * @example
     * import { RandomForestClassifier } from 'kalimdor/ensemble';
     *
     * const X = [[0, 0], [1, 1], [2, 1], [1, 5], [3, 2]];
     * const y = [0, 1, 2, 3, 7];
     *
     * const randomForest = new RandomForestClassifier();
     * randomForest.fit({ X, y });
     *
     * // Results in a value such as [ '0', '2' ].
     * // Predictions will change as we have not set a seed value.
     */
    export class RandomForestClassifier extends BaseRandomForest {
        /**
         * Predict class for X.
         *
         * The predicted class of an input sample is a vote by the trees in the forest, weighted by their probability estimates.
         * That is, the predicted class is the one with highest mean probability estimate across the trees.
         * @param {Array} X - array-like or sparse matrix of shape = [n_samples]
         * @returns {string[]}
         */
        predict(X?: number[] | number[][]): any[];
        /**
         * @hidden
         * Bagging prediction helper method
         * According to the predictions returns by the trees, it will select the
         * class with the maximum number (votes)
         * @param {Array<any>} predictions - List of initial predictions that may look like [ [1, 2], [1, 1] ... ]
         * @returns {string[]}
         */
        private votePredictions;
    }
}
declare module "lib/ensemble/index" {
    import { RandomForestClassifier } from "lib/ensemble/forest";
    export { RandomForestClassifier };
}
declare module "lib/utils/nlp" {
    /**
     * @ignore
     */
    export class WordTokenizer {
        /**
         * Tokenize a given text
         * e.g.
         * given: "deep-learning ian good fellow learning jason shin shin"
         * returns: [ 'deep', 'learning', 'ian', 'good', 'fellow', 'learning', 'jason', 'shin', 'shin' ]
         * @param text
         * @returns {string[]}
         */
        tokenize(text: any): string[];
    }
}
declare module "lib/feature_extraction/stop_words" {
    /**
     * @ignore
     * @type {Array<string>}
     */
    export const ENGLISH_STOP_WORDS: string[];
}
declare module "lib/feature_extraction/text" {
    /**
     * The CountVectorizer provides a simple way to both tokenize a collection
     * of text documents and build a vocabulary of known words, but also
     * to encode new documents using that vocabulary.
     *
     * @example
     * import { CountVectorizer } from 'kalimdor/feature_extraction';
     *
     * const corpus = ['deep learning ian good fellow learning jason shin shin', 'yoshua bengio'];
     * const vocabCounts = cv.fit_transform(corpus);
     * console.log(vocabCounts); // [ [ 0, 1, 1, 1, 1, 1, 2, 2, 0 ], [ 1, 0, 0, 0, 0, 0, 0, 0, 1 ] ]
     * console.log(cv.vocabulary); // { bengio: 0, deep: 1, fellow: 2, good: 3, ian: 4, jason: 5, learning: 6, shin: 7, yoshua: 8 }
     * console.log(cv.getFeatureNames()); // [ 'bengio', 'deep', 'fellow', 'good', 'ian', 'jason', 'learning', 'shin', 'yoshua' ]
     *
     * const newVocabCounts = cv.transform(['ian good fellow jason duuog']);
     * console.log(newVocabCounts); // [ [ 0, 0, 1, 1, 1, 1, 0, 0, 0 ] ]
     */
    export class CountVectorizer {
        vocabulary: object;
        /** @ignore */
        private internalVocabulary;
        /**
         * Learn a vocabulary dictionary of all tokens in the raw documents.
         * @param {string[]} doc - An array of strings
         * @returns {CountVectorizer}
         */
        fit(doc?: string[]): this;
        /**
         * fit transform applies
         * @param {string[]} doc - An array of strings
         * @returns {number[][]}
         */
        fit_transform(doc?: string[]): number[][];
        /**
         * Transform documents to document-term matrix.
         * Extract token counts out of raw text documents using the vocabulary
         * fitted with fit or the one provided to the constructor.
         * @param {string[]} doc - An array of strings
         * @returns {number[][]}
         */
        transform(doc?: string[]): number[][];
        /**
         * Array mapping from feature integer indices to feature name
         * @returns {Object}
         */
        getFeatureNames(): object;
        /**
         * Build a tokenizer/vectorizer
         * @returns {(x: string) => string[]}
         */
        private buildAnalyzer;
        /**
         * Calculates list of vocabularies in the entire document and come up with
         * vocab: index pairs
         * @param doc
         */
        private buildVocabulary;
        /**
         * @ignore
         * Counting number of vocab occurences in the current token of a sentence
         * ['yoshua', 'bengio', 'deep', 'learning'] = vocabulary
         * ['yohua', 'bengio'] => tokens
         * results in
         * [1, 1, 0, 0]
         * @param doc
         */
        private countVocab;
        /**
         * @ignore
         * preprocess a line of text by applying
         * 1) tokenization
         * 2) removing stopwords
         * @param text
         * @param {any} removeSW
         * @returns {any}
         */
        private preprocess;
    }
}
declare module "lib/feature_extraction/index" {
    import { CountVectorizer } from "lib/feature_extraction/text";
    export { CountVectorizer };
}
declare module "lib/linear_model/linear_regression" {
    /**
     * Ordinary least squares Linear Regression.
     *
     * @example
     * import { LinearRegression } from './linear_regression';
     * const linearRegression = new LinearRegression();
     * const X = [1, 2, 4, 3, 5];
     * const y = [1, 3, 3, 2, 5];
     * linearRegression.fit({ X, y });
     * console.log(lr.predict([1, 2]));
     * // [ 1.1999999999999995, 1.9999999999999996 ]
     */
    export class LinearRegression {
        private b0;
        private b1;
        /**
         * fit linear model
         * @param {any} X - training values
         * @param {any} y - target values
         */
        fit({ X, y }: {
            X: number[];
            y: number[];
        }): void;
        /**
         * Predict using the linear model
         * @param {number} X - Values to predict.
         * @returns {number}
         */
        predict(X?: number[]): number[];
        /**
         * Get the model details in JSON format
         */
        toJSON(): {
            /**
             * Coefficients component: b0
             */
            b0: number;
            /**
             * Coefficients component: b1
             */
            b1: number;
        };
        /**
         * Restore the model from a checkpoint
         * @param {number} b0 - Coefficients component: b0
         * @param {number} b1 - Coefficients component: b1
         */
        fromJSON({ b0, b1 }: {
            b0: number;
            b1: number;
        }): void;
        /**
         * Calculates coefficient for linear regression
         * @param X - X values
         * @param y - y targets
         */
        private coefficients;
    }
}
declare module "lib/utils/permutations" {
    /**
     * Generate n combinations with repeat values.
     * @param X - Matrix input
     * @param n - number of repeats
     * @ignore
     */
    export function combinationsWithReplacement(X: any, n?: number): number[][];
}
declare module "lib/preprocessing/data" {
    /**
     * Augment dataset with an additional dummy feature.
     * This is useful for fitting an intercept term with implementations which cannot otherwise fit it directly.
     *
     * @example
     * import { add_dummy_feature } from 'kalimdor/preprocessing';
     * const dummy = add_dummy_feature({ X: [[0, 1, 2], [1, 0, 3]] });
     * console.log(dummy); // returns: [ [ 1, 0, 1, 2 ], [ 1, 1, 0, 3 ] ]
     *
     * @param X - A matrix of data
     * @param value - Value to use for the dummy feature.
     */
    export function add_dummy_feature({ X, value }?: {
        X: number[][];
        value?: number;
    }): number[][];
    /**
     * Encode categorical integer features using a one-hot aka one-of-K scheme.
     *
     * The input to this transformer should be a matrix of integers, denoting the
     * values taken on by categorical (discrete) features. The output will be a sparse
     * matrix where each column corresponds to one possible value of one feature.
     * It is assumed that input features take on values in the range [0, n_values).
     *
     * This encoding is needed for feeding categorical data to many
     * scikit-learn estimators, notably linear models and SVMs with the standard kernels.
     *
     * Note: a one-hot encoding of y labels should use a LabelBinarizer instead.
     *
     * @example
     * const enc = new OneHotEncoder();
     * const planetList = [
     *  { planet: 'mars', isGasGiant: false, value: 10 },
     *  { planet: 'saturn', isGasGiant: true, value: 20 },
     *  { planet: 'jupiter', isGasGiant: true, value: 30 }
     * ];
     * const encodeInfo = enc.encode(planetList, {
     *  dataKeys: ['value', 'isGasGiant'],
     *  labelKeys: ['planet']
     * });
     * // encodeInfo.data -> [ [ -1, 0, 1, 0, 0 ], [ 0, 1, 0, 1, 0 ], [ 1, 1, 0, 0, 1 ] ]
     * const decodedInfo = enc.decode(encodeInfo.data, encodeInfo.decoders);
     * // gives you back the original value, which is `planetList`
     */
    export class OneHotEncoder {
        /**
         * encode data according to dataKeys and labelKeys
         *
         * @param data - list of records to encode
         * @param options - dataKeys: independent variables, labelKeys: dependent variables; mandatory
         * @return {{data: Array, decoders: Array}} - see readme for full explanation
         */
        encode(data: any, options?: {
            dataKeys: any;
            labelKeys: any;
        }): {
            data: any[];
            decoders: any[];
        };
        /**
         * Decode the encoded data back into its original format
         */
        decode(encoded: any, decoders: any): any[];
        /**
         * Decode an encoded row back into its original format
         * @param row
         * @param decoders
         * @returns {Object}
         */
        private decodeRow;
        /**
         * Standardizing field
         * Example dataset:
         * [ { planet: 'mars', isGasGiant: false, value: 10 },
         * { planet: 'saturn', isGasGiant: true, value: 20 },
         * { planet: 'jupiter', isGasGiant: true, value: 30 } ]
         *
         * @param key: each key/feature such as planet, isGasGiant and value
         * @param data: the entire dataset
         * @returns {any}
         */
        private standardizeField;
        /**
         * Calculating the sample standard deviation (vs population stddev).
         * @param lst
         * @param {number} mean
         * @returns {number}
         */
        private calculateStd;
        /**
         * One hot encode a number value
         *
         * @param type
         * @param key
         * @param values
         * @returns {{encoded: any[]; decode: {type: any; mean: number; std: number; key: any}}}
         */
        private buildNumberOneHot;
        /**
         * One hot encode a boolean value
         *
         * Example usage:
         * boolEncoder.encode(true) => 1
         * boolEncoder.encode(false) => 0
         *
         * @param type
         * @param key
         * @param values
         * @returns {{encode}}
         */
        private buildBooleanOneHot;
        /**
         * One hot encode a string value
         *
         * Example for internal reference (unnecessary details for those just using this module)
         *
         * const encoder = buildOneHot(['RAIN', 'RAIN', 'SUN'])
         * // encoder == { encode: () => ... , lookupTable: ['RAIN', 'SUN'] }
         * encoder.encode('SUN')  // [0, 1]
         * encoder.encode('RAIN') // [1, 0]
         * encoder.encode('SUN')  // [1, 0]
         * // encoder.lookupTable can then be passed into this.decode to translate [0, 1] back into 'SUN'
         *
         * It's not ideal (ideally it would all just be done in-memory and we could return a "decode" closure,
         * but it needs to be serializable to plain old JSON.
         */
        private buildStringOneHot;
    }
    /**
     * Transforms features by scaling each feature to a given range.
     *
     * This estimator scales and translates each feature individually such that it is in the given range on the training set, i.e. between zero and one.
     *
     * The transformation is given by:
     *
     * ```
     * X_std = (X - X.min(axis=0)) / (X.max(axis=0) - X.min(axis=0))
     * X_scaled = X_std * (max - min) + min
     * ```
     *
     * where min, max = feature_range.
     *
     * This transformation is often used as an alternative to zero mean, unit variance scaling.
     *
     * @example
     * import { MinMaxScaler } from 'kalimdor/preprocessing';
     *
     * const minmaxScaler = new MinMaxScaler({ featureRange: [0, 1] });
     * minmaxScaler.fit([4, 5, 6]);
     * const result = minmaxScaler.fit_transform([4, 5, 6]);
     * // [ 0, 0.5, 1 ]
     */
    export class MinMaxScaler {
        private featureRange;
        private clone;
        private dataMax;
        private dataMin;
        private featureMax;
        private featureMin;
        private dataRange;
        private scale;
        private baseMin;
        /**
         * @param featureRange - scaling range
         * @param clone - to clone the input
         */
        constructor({ featureRange, clone }?: {
            featureRange?: number[];
            clone?: boolean;
        });
        /**
         * Compute the minimum and maximum to be used for later scaling.
         * @param {number[]} X - Array or sparse-matrix data input
         */
        fit(X: number[] | number[][]): void;
        /**
         * Fit to data, then transform it.
         * @param {number[]} X - Original input vector
         */
        fit_transform(X: number[]): number[];
        /**
         * Undo the scaling of X according to feature_range.
         * @param {number[]} X - Scaled input vector
         */
        inverse_transform(X: number[]): number[];
    }
    /**
     * Binarizer transform your data using a binary threshold.
     * All values above the threshold are marked 1 and all equal to or below are marked as 0.
     *
     * It can also be used as a pre-processing step for estimators that consider
     * boolean random variables (e.g. modelled using the Bernoulli distribution in
     * a Bayesian setting).
     *
     * @example
     * import { Binarizer } from 'kalimdor/preprocessing';
     *
     * const binX = [[1, -1, 2], [2, 0, 0], [0, 1, -1]];
     * const binarizer = new Binarizer({ threshold: 0 });
     * const result = binarizer.transform(binX);
     * // [ [ 1, 0, 1 ], [ 1, 0, 0 ], [ 0, 1, 0 ] ]
     */
    export class Binarizer {
        private threshold;
        private copy;
        /**
         *
         * @param {number} threshold - Feature values below or equal to this are replaced by 0, above it by 1.
         * @param {boolean} copy - Flag to clone the input value.
         */
        constructor({ copy, threshold }?: {
            copy?: boolean;
            threshold?: number;
        });
        /**
         * Currently fit does nothing
         * @param {any[]} X - Does nothing
         */
        fit(X?: any[]): void;
        /**
         * Transforms matrix into binarized form
         * X = [[ 1., -1.,  2.],
         *      [ 2.,  0.,  0.],
         *      [ 0.,  1., -1.]]
         * becomes
         * array([[ 1.,  0.,  1.],
         *    [ 1.,  0.,  0.],
         *    [ 0.,  1.,  0.]])
         * @param {any[]} X - The data to binarize.
         */
        transform(X?: any[]): any[];
    }
    /**
     * Generate polynomial and interaction features.
     *
     * Generate a new feature matrix consisting of all polynomial combinations of the features
     * with degree less than or equal to the specified degree. For example, if an input sample
     * is two dimensional and of the form [a, b], the degree-2 polynomial features are [1, a, b, a^2, ab, b^2].
     *
     * @example
     * import { PolynomialFeatures } from 'kalimdor/preprocessing';
     * const poly = new PolynomialFeatures();
     * const X = [[0, 1], [2, 3], [4, 5]];
     * poly.transform({ X });
     * // Result:
     * // [ [ 1, 0, 1, 0, 0, 1 ],
     * // [ 1, 2, 3, 4, 6, 9 ],
     * // [ 1, 4, 5, 16, 20, 25 ] ]
     *
     */
    export class PolynomialFeatures {
        private degree;
        /**
         *
         * @param degree - The degree of the polynomial features. Default = 2.
         */
        constructor({ degree }?: {
            degree: number;
        });
        /**
         * Transforms the input data
         * @param X - a matrix
         */
        transform({ X }?: {
            X: number[][];
        }): number[][];
        /**
         * Creates a combination of index according to nFeautres and degree
         * @param nFeatures
         * @param degree
         */
        private indexCombination;
    }
    /**
     * Data normalization is a process of scaling dataset based on Vector Space Model, and by default, it uses L2 normalization.
     * At a higher level, the chief difference between the L1 and the L2 terms is that the L2 term is proportional
     * to the square of the  β values, while the L1 norm is proportional the absolute value of the values in  β .
     *
     * @example
     * import { normalize } from 'kalimdor/preprocessing';
     *
     * const result = normalize({
     *   X: [
     *     [1, -1, 2],
     *     [2, 0, 0],
     *     [0, 1, -1],
     *   ],
     * });
     * console.log(result);
     * // [ [ 0.4082482904638631, -0.4082482904638631, 0.8164965809277261 ],
     * // [ 1, 0, 0 ],
     * // [ 0, 0.7071067811865475, -0.7071067811865475 ] ]
     *
     * @param X - The data to normalize
     * @param norm - The norm to use to normalize each non zero sample; can be either 'l1' or 'l2'
     * @return number[][]
     */
    export function normalize({ X, norm }?: {
        X: number[][];
        norm?: string;
    }): number[][];
}
declare module "lib/utils/validation" {
    /**
     * Check below array conditions
     * - multiclass
     *    - e.g. [ [1, 2], [2, 3] ]
     *      Then it sets multiclass value to true
     * - isArray<boolean>
     *   If the given arr is an array then the value is true else false
     * @param arr
     * @returns {any}
     * @ignore
     */
    export function checkArray(arr: any[]): {
        readonly isArray: boolean;
        readonly multiclass: boolean;
    };
}
declare module "lib/preprocessing/Imputer" {
    /**
     * Imputation transformer for completing missing values.
     *
     * @example
     * import { Imputer } from 'preprocessing/Imputer';
     *
     * const testX = [[1, 2], [null, 3], [7, 6]];
     * const imp = new Imputer({ missingValues: null, axis: 0 });
     * imp.fit(testX);
     * const impResult = imp.fit_transform([[null, 2], [6, null], [7, 6]]);
     * // [ [ 4, 2 ], [ 6, 3.6666666666666665 ], [ 7, 6 ] ]
     */
    export class Imputer {
        private missingValues;
        private strategy;
        private axis;
        private copy;
        private means;
        /**
         *
         * @param {any} missingValues - Target missing value to impute
         * @param {any} strategy - Missing value replacement strategy
         * @param {any} axis - Direction to impute
         * @param {any} copy - To clone the input value
         */
        constructor({ missingValues, strategy, axis, copy }: {
            missingValues?: any;
            strategy?: string;
            axis?: number;
            copy?: boolean;
        });
        /**
         * Fit the imputer on X.
         * @param {any[]} X - Input data in array or sparse matrix format
         */
        fit(X?: any[]): void;
        /**
         * Fit to data, then transform it.
         * @param {any[]} X - Input data in array or sparse matrix format
         * @returns {any[]}
         */
        fit_transform(X?: any[]): any[];
        /**
         * Calculate array of numbers as array of mean values
         * Examples:
         * [ [ 1, 2 ], [ null, 3 ], [ 123, 3 ] ]
         * => [ 1.5, 3, 63 ]
         *
         * [ [ 1, 123 ], [ 2, 3, 3 ] ]
         * => [ 62, 2.6666666666666665 ]
         *
         * @param matrix
         * @param {string[]} steps
         */
        private calcArrayMean;
    }
}
declare module "lib/preprocessing/index" {
    import { add_dummy_feature, Binarizer, MinMaxScaler, normalize, OneHotEncoder, PolynomialFeatures } from "lib/preprocessing/data";
    import { Imputer } from "lib/preprocessing/Imputer";
    export { add_dummy_feature, Binarizer, MinMaxScaler, normalize, OneHotEncoder, PolynomialFeatures, Imputer };
}
declare module "lib/linear_model/stochastic_gradient" {
    /**
     * Ordinary base class for SGD classier or regressor
     * @ignore
     */
    class BaseSGD {
        protected scaler: any;
        private preprocess;
        private learningRate;
        private epochs;
        private coefficients;
        private clone;
        /**
         * @param preprocess - preprocess methodology can be either minmax or null. Default is minmax.
         * @param learning_rate - Used to limit the amount each coefficient is corrected each time it is updated.
         * @param epochs - Number of iterations.
         * @param clone - To clone the passed in dataset.
         */
        constructor({ preprocess, learning_rate, epochs, clone }?: {
            preprocess?: string;
            learning_rate?: number;
            epochs?: number;
            clone?: boolean;
        });
        /**
         * Train the base SGD
         * @param X - Matrix of data
         * @param y - Matrix of targets
         */
        fit({ X, y }?: {
            X: number[][];
            y: number[];
        }): void;
        /**
         * Save the model's checkpoint
         */
        toJSON(): {
            /**
             * coefficient values
             */
            coefficients: number[];
            /**
             * scaler object such as MinMaxScaler
             */
            scaler: {};
        };
        /**
         * Restore the model from a checkpoint
         * @param coefficients - coefficient values
         * @param scaler - scaler object such as MinMaxScaler
         */
        fromJSON({ coefficients, scaler }?: {
            coefficients: number[];
            scaler: {};
        }): void;
        /**
         * Predictions according to the passed in test set
         * @param X - Matrix of data
         */
        protected predict({ X }: {
            X: any;
        }): number[];
        /**
         * Calculate yhat of a row in the dataset
         * @param row
         * @param coefficients
         */
        private predictOne;
        /**
         * SGD based on linear model to calculate coefficient
         * @param X - training data
         * @param y - target data
         */
        private sgd;
    }
    /**
     * Linear classifiers (SVM, logistic regression, a.o.) with SGD training.
     *
     * This estimator implements regularized linear models with
     * stochastic gradient descent (SGD) learning: the gradient of
     * the loss is estimated each sample at a time and the model is
     * updated along the way with a decreasing strength schedule
     * (aka learning rate). SGD allows minibatch (online/out-of-core)
     * learning, see the partial_fit method. For best results using
     * the default learning rate schedule, the data should have zero mean
     * and unit variance.
     *
     * @example
     * import { SGDClassifier } from 'kalimdor/linear_model';
     * const clf = new SGDClassifier();
     * const X = [[0., 0.], [1., 1.]];
     * const y = [0, 1];
     * clf.fit({ X, y });
     * clf.predict({ X: [[2., 2.]] }); // result: [ 1 ]
     *
     */
    export class SGDClassifier extends BaseSGD {
        /**
         * Predicted values with Math.round applied
         * @param X - Matrix of data
         */
        predict({ X }?: {
            X: number[][];
        }): number[];
    }
    /**
     * Linear model fitted by minimizing a regularized empirical loss with SGD
     * SGD stands for Stochastic Gradient Descent: the gradient of the loss
     * is estimated each sample at a time and the model is updated along
     * the way with a decreasing strength schedule (aka learning rate).
     *
     * @example
     * import { SGDRegressor } from 'kalimdor/linear_model';
     * const reg = new SGDRegressor();
     * const X = [[0., 0.], [1., 1.]];
     * const y = [0, 1];
     * reg.fit({ X, y });
     * reg.predict({ X: [[2., 2.]] }); // result: [ 1.281828588248001 ]
     *
     */
    export class SGDRegressor extends BaseSGD {
        /**
         * Predicted values
         * @param X - Matrix of data
         */
        predict({ X }?: {
            X: number[][];
        }): number[];
    }
}
declare module "lib/linear_model/index" {
    import { LinearRegression } from "lib/linear_model/linear_regression";
    import { SGDClassifier, SGDRegressor } from "lib/linear_model/stochastic_gradient";
    export { LinearRegression, SGDClassifier, SGDRegressor };
}
declare module "lib/metrics/classification" {
    /**
     * Validator for classification exceptions
     * @param y_true
     * @param y_pred
     * @param labels
     * @param options
     * @ignore
     */
    export const validateInitialInputs: (y_true: any, y_pred: any, labels: any, options?: {}) => void;
    /**
     * Accuracy classification score.
     *
     * In multilabel classification, this function computes subset accuracy:
     * the set of labels predicted for a sample must exactly match the corresponding set of labels in y_true.
     *
     * @example
     * import { accuracyScore } from 'kalimdor/metrics';
     *
     * const accResult = accuracyScore({
     *   y_true: [0, 1, 2, 3],
     *   y_pred: [0, 2, 1, 3]
     * });
     *
     * // accuracy result: 0.5
     *
     * @param {any} y_true
     * @param {any} y_pred
     * @param {any} normalize
     * @param {any} sample_weight
     */
    export function accuracyScore({ y_true, y_pred, normalize }: {
        y_true: any;
        y_pred: any;
        normalize?: boolean;
    }): number;
    /**
     * Zero-one classification loss.
     *
     * If normalize is `true`, return the fraction of misclassifications (float),
     * else it returns the number of misclassifications (int). The best performance is 0.
     *
     * @example
     * import { zeroOneLoss } from 'kalimdor/metrics';
     *
     * const loss_zero_one_result = zeroOneLoss({
     *   y_true: [1, 2, 3, 4],
     *   y_pred: [2, 2, 3, 5]
     * });
     * console.log(loss_zero_one_result); // 0.5
     *
     * @param {any} y_true
     * @param {any} y_pred
     * @param {any} normalize
     * @returns {number}
     */
    export function zeroOneLoss({ y_true, y_pred, normalize }: {
        y_true: any;
        y_pred: any;
        normalize?: boolean;
    }): number;
    export interface ConfusionMatrixOptions {
        /**
         * Ground truth (correct) target values.
         */
        y_true: any[];
        /**
         * Estimated targets as returned by a classifier.
         */
        y_pred: any[];
        /**
         * List of labels to index the matrix. This may be used to reorder or
         * select a subset of labels. If none is given, those that appear
         * at least once in y_true or y_pred are used in sorted order.
         */
        labels?: any[];
    }
    /**
     * A confusion matrix is a technique for summarizing the performance of a classification algorithm.
     *
     * Classification accuracy alone can be misleading if you have an unequal number of observations in each class or if you have more than two classes in your dataset.
     *
     * Calculating a confusion matrix can give you a better idea of what your classification model is getting right and what types of errors it is making.
     *
     * @example
     * import { confusion_matrix } from 'kalimdor/metrics';
     *
     * const matrix1 = confusion_matrix({
     *   y_true: [1, 2, 3],
     *   y_pred: [1, 2, 3]
     * });
     * console.log(matrix1); // [ [ 1, 0, 0 ], [ 0, 1, 0 ], [ 0, 0, 1 ] ]
     *
     * const matrix2 = confusion_matrix({
     *   y_true: ['cat', 'ant', 'cat', 'cat', 'ant', 'bird'],
     *   y_pred: ['ant', 'ant', 'cat', 'cat', 'ant', 'cat']
     * });
     * console.log(matrix2); // [ [ 1, 2, 0 ], [ 2, 0, 0 ], [ 0, 1, 0 ] ]
     *
     * @param {ConfusionMatrixOptions} options
     * @returns {number[]}
     */
    export function confusion_matrix(options: ConfusionMatrixOptions): number[];
}
declare module "lib/metrics/index" {
    import { accuracyScore, confusion_matrix, zeroOneLoss } from "lib/metrics/classification";
    export { accuracyScore, confusion_matrix, zeroOneLoss };
}
declare module "lib/model_selection/_split" {
    /**
     * K-Folds cross-validator
     *
     * Provides train/test indices to split data in train/test sets. Split dataset into k consecutive folds (without shuffling by default).
     *
     * Each fold is then used once as a validation while the k - 1 remaining folds form the training set.
     *
     * @example
     * import { KFold } from 'kalimdor/model_selection';
     *
     * const kFold = new KFold({ k: 5 });
     * const X1 = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1];
     * console.log(kFold.split({ X: X1, y: X1 }));
     *
     * /* [ { trainIndex: [ 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19 ],
     * *  testIndex: [ 0, 1, 2, 3 ] },
     * * { trainIndex: [ 0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19 ],
     * *  testIndex: [ 4, 5, 6, 7 ] },
     * * { trainIndex: [ 0, 1, 2, 3, 4, 5, 6, 7, 12, 13, 14, 15, 16, 17, 18, 19 ],
     * *  testIndex: [ 8, 9, 10, 11 ] },
     * * { trainIndex: [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 16, 17, 18, 19 ],
     * *  testIndex: [ 12, 13, 14, 15 ] },
     * * { trainIndex: [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 ],
     * *  testIndex: [ 16, 17, 18, 19 ] } ]
     *
     */
    export class KFold {
        private k;
        private shuffle;
        /**
         *
         * @param {any} k - Number of folds. Must be at least 2.
         * @param {any} shuffle - Whether to shuffle the data before splitting into batches.
         */
        constructor({ k, shuffle }: {
            k?: number;
            shuffle?: boolean;
        });
        /**
         *
         * @param {any} X - Training data, where n_samples is the number of samples and n_features is the number of features.
         * @param {any} y - The target variable for supervised learning problems.
         * @returns {any[]}
         */
        split({ X, y }: {
            X: any;
            y: any;
        }): any[];
    }
    /**
     * Split arrays or matrices into random train and test subsets
     *
     * @example
     * import { train_test_split } from 'kalimdor/model_selection';
     *
     * const X = [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9]];
     * const y = [0, 1, 2, 3, 4];
     *
     * train_test_split({
     *   X,
     *   y,
     *   test_size: 0.33,
     *   train_size: 0.67,
     *   random_state: 42
     * });
     *
     * /*
     * * { xTest: [ [ 0, 1 ], [ 8, 9 ] ],
     * *  xTrain: [ [ 4, 5 ], [ 6, 7 ], [ 2, 3 ] ],
     * *  yTest: [ 0, 4 ],
     * *  yTrain: [ 2, 3, 1 ] }
     *
     * @param {any} X - input data
     * @param {any} y - target data
     * @param {number} test_size - size of the returning test set
     * @param {number} train_size - size of the returning training set
     * @param {number} random_state - state used to shuffle data
     * @param {boolean} clone - to clone the original data
     * @returns {{xTest: any[]; xTrain: any[]; yTest: any[]; yTrain: any[]}}
     */
    export function train_test_split({ X, y, test_size, train_size, random_state, clone }?: {
        X: any[];
        y: any[];
        test_size?: number;
        train_size?: number;
        random_state?: number;
        clone?: boolean;
    }): {
        xTest: any[];
        xTrain: any[];
        yTest: any[];
        yTrain: any[];
    };
}
declare module "lib/model_selection/index" {
    import { KFold, train_test_split } from "lib/model_selection/_split";
    export { KFold, train_test_split };
}
declare module "lib/neighbors/KDTree" {
    /**
     * @ignore
     */
    export class Node {
        obj: any;
        left: any;
        right: any;
        parent: any;
        dimension: any;
        constructor(obj: any, dimension: any, parent: any);
    }
    /**
     * @ignore
     */
    export class KDTree {
        dimensions: any;
        root: any;
        private metric;
        constructor(points: any, metric: any);
        toJSON(): Node;
        nearest(point: any, maxNodes: any, maxDistance: any): any[];
    }
}
declare module "lib/neighbors/classification" {
    /**
     * Classifier implementing the k-nearest neighbors vote.
     *
     * @example
     * const knn = new KNeighborsClassifier();
     * const X = [[0, 0, 0], [0, 1, 1], [1, 1, 0], [2, 2, 2], [1, 2, 2], [2, 1, 2]];
     * const y = [0, 0, 0, 1, 1, 1];
     * knn.fit({ X, y });
     * console.log(knn.predict([1, 2])); // predicts 1
     */
    export class KNeighborsClassifier {
        private type;
        private tree;
        private k;
        private classes;
        private distance;
        /**
         * @param {string} distance - Choice of distance function, should choose between euclidean | manhattan
         * @param {number} k - Number of neighbors to classify
         * @param {string} type - Type of algorithm to use, choose between kdtree(default) | balltree | simple
         */
        constructor({ distance, k, type }?: {
            distance: string;
            k: number;
            type: string;
        });
        /**
         * Train the classifier with input and output data
         * @param {any} X - Training data.
         * @param {any} y - Target data.
         */
        fit({ X, y }: {
            X: number[][];
            y: number[];
        }): void;
        /**
         * Return the model's state as a JSON object
         * @return {object} JSON KNN model.
         */
        toJSON(): {
            classes: any[];
            distance: any;
            k: number;
            tree: any;
            type: string;
        };
        /**
         * Restores the model from a JSON checkpoint
         * @param {any} classes
         * @param {any} distance
         * @param {any} k
         * @param {any} tree
         * @param {any} type
         */
        fromJSON({ classes, distance, k, tree, type }: {
            classes?: any;
            distance?: any;
            k?: any;
            tree?: any;
            type?: any;
        }): void;
        /**
         * Predict single value from a list of data
         * @param {Array} X - Prediction data.
         * @returns number
         */
        predict(X?: any): any;
        /**
         * Runs a single prediction against an array based on kdTree or balltree or
         * simple algo
         * @param array
         * @returns {{}}
         */
        private getSinglePred;
        /**
         * Get the class with the max point
         * @param current
         * @returns {{}}
         * @ignore
         */
        private getTreeBasedPrediction;
    }
}
declare module "lib/neighbors/index" {
    import { KNeighborsClassifier } from "lib/neighbors/classification";
    export { KNeighborsClassifier };
}
declare module "lib/svm/classes" {
    export type Type = 'C_SVC' | 'NU_SVC' | 'ONE_CLASS' | 'EPSILON_SVR' | 'NU_SVR';
    export type Kernel = 'LINEAR' | 'POLYNOMIAL' | 'RBF' | 'SIGMOID';
    /**
     * Options used by sub classes
     * Notice type is disabled as they are set statically from children classes
     */
    export interface SVMOptions {
        /**
         * Type of SVM
         */
        type?: Type;
        /**
         * Degree of polynomial, test for polynomial kernel
         */
        degree?: number;
        /**
         * Type of Kernel
         */
        kernel?: Kernel;
        /**
         * Gamma parameter of the RBF, Polynomial and Sigmoid kernels. Default value is 1/num_features
         */
        gamma?: number | null;
        /**
         * coef0 parameter for Polynomial and Sigmoid kernels
         */
        coef0?: number;
        /**
         * Cost parameter, for C SVC, Epsilon SVR and NU SVR
         */
        cost?: number;
        /**
         * For NU SVC and NU SVR
         */
        nu?: number;
        /**
         * For epsilon SVR
         */
        epsilon?: number;
        /**
         * Cache size in MB
         */
        cacheSize?: number;
        /**
         * Tolerance
         */
        tolerance?: number;
        /**
         * Use shrinking euristics (faster)
         */
        shrinking?: boolean;
        /**
         * weather to train SVC/SVR model for probability estimates,
         */
        probabilityEstimates?: boolean;
        /**
         * Set weight for each possible class
         */
        weight?: object | null;
        /**
         * Print info during training if false (aka verbose)
         */
        quiet?: boolean;
    }
    /**
     * BaseSVM class used by all parent SVM classes that are based on libsvm
     */
    export class BaseSVM {
        static create(options?: SVMOptions): Promise<BaseSVM>;
        /**
         * Get Kernel name type using string Kernel name
         * @param SVM
         * @param {string} name
         * @returns {number}
         */
        private static getKernel;
        /**
         * Get Kernel type using string type name
         * @param SVM
         * @param {string} name
         * @returns {number}
         */
        private static getType;
        /**
         * Get a consolidated options including type and Kernel
         * @param SVM
         * @param {Options} options
         * @param {Type} type
         * @param {Kernel} kernel
         * @returns {Object}
         */
        private static processOptions;
        protected svm: any;
        protected options: SVMOptions;
        protected constructor(svm: any, options: SVMOptions);
        /**
         * Fit the model according to the given training data.
         * @param {number[][]} X
         * @param {number[]} y
         * @returns {Promise<void>}
         */
        fit({ X, y }: {
            X: number[][];
            y: number[];
        }): Promise<void>;
        /**
         * Predict using the linear model
         * @param {number[][]} X
         * @returns {number[]}
         */
        predict(X: number[][]): number[];
        /**
         * Predict the label of one sample.
         * @param {number[]} X
         * @returns {number}
         */
        predictOne(X: number[]): number;
        /**
         * Saves the current SVM as a JSON object
         * @returns {{svm: string; options: SVMOptions}}
         */
        toJSON(): {
            svm: any;
            options: SVMOptions;
        };
        /**
         * Restores the model from a JSON checkpoint
         * @param {any} svm
         * @param {any} type
         * @param {any} options
         */
        fromJSON(serialized: any): Promise<void>;
    }
    /**
     * C-Support Vector Classification.
     *
     * The implementation is based on libsvm. The fit time complexity is more than
     * quadratic with the number of samples which makes it hard to scale to dataset
     * with more than a couple of 10000 samples.
     *
     * The multiclass support is handled according to a one-vs-one scheme.
     *
     * For details on the precise mathematical formulation of the provided kernel
     * functions and how gamma, coef0 and degree affect each other, see the corresponding
     * section in the narrative documentation: Kernel functions.
     */
    export class SVC extends BaseSVM {
        static create(options?: SVMOptions): Promise<SVC>;
    }
    /**
     * Linear Support Vector Regression.
     *
     * Similar to SVR with parameter kernel=’linear’, but implemented in terms of
     * liblinear rather than libsvm, so it has more flexibility in the choice of
     * penalties and loss functions and should scale better to large numbers of samples.
     *
     * This class supports both dense and sparse input.
     */
    export class SVR extends BaseSVM {
        static create(options?: SVMOptions): Promise<SVR>;
    }
    /**
     * Unsupervised Outlier Detection.
     *
     * Estimate the support of a high-dimensional distribution.
     *
     * The implementation is based on libsvm.
     */
    export class OneClassSVM extends BaseSVM {
        static create(options?: SVMOptions): Promise<OneClassSVM>;
    }
    /**
     * Nu-Support Vector Classification.
     *
     * Similar to SVC but uses a parameter to control the number of support vectors.
     *
     * The implementation is based on libsvm.
     */
    export class NuSVC extends BaseSVM {
        static create(options?: SVMOptions): Promise<NuSVC>;
    }
    /**
     * Nu Support Vector Regression.
     *
     * Similar to NuSVC, for regression, uses a parameter nu to control the number
     * of support vectors. However, unlike NuSVC, where nu replaces C, here nu
     * replaces the parameter epsilon of epsilon-SVR.
     *
     * The implementation is based on libsvm.
     */
    export class NuSVR extends BaseSVM {
        static create(options?: SVMOptions): Promise<NuSVR>;
    }
}
declare module "lib/svm/index" {
    import { BaseSVM, NuSVC, NuSVR, OneClassSVM, SVC, SVR } from "lib/svm/classes";
    export { BaseSVM, NuSVC, NuSVR, OneClassSVM, SVC, SVR };
}
declare module "lib/tree/index" {
    import { DecisionTreeClassifier } from "lib/tree/tree";
    export { DecisionTreeClassifier };
}
declare module "lib/index" {
    import * as cluster from "lib/cluster/index";
    import * as datasets from "lib/datasets/index";
    import * as decomposition from "lib/decomposition/index";
    import * as ensemble from "lib/ensemble/index";
    import * as feature_extraction from "lib/feature_extraction/index";
    import * as linear_model from "lib/linear_model/index";
    import * as metrics from "lib/metrics/index";
    import * as model_selection from "lib/model_selection/index";
    import * as neighbors from "lib/neighbors/index";
    import * as preprocessing from "lib/preprocessing/index";
    import * as svm from "lib/svm/index";
    import * as tree from "lib/tree/index";
    export { cluster, datasets, decomposition, ensemble, feature_extraction, linear_model, metrics, model_selection, neighbors, preprocessing, svm, tree, };
}
declare module "lib/cluster/index.repl" { }
declare module "lib/datasets/index.repl" { }
declare module "lib/decomposition/index.repl" { }
declare module "lib/ensemble/index.repl" { }
declare module "lib/feature_extraction/index.repl" { }
declare module "lib/linear_model/index.repl" { }
declare module "lib/metrics/index.repl" { }
declare module "lib/model_selection/index.repl" { }
declare module "lib/naive_bayes/gaussian" {
    /**
     * The Naive is an intuitive method that uses probabilistic of each attribute
     * belonged to each class to make a prediction. It uses Gaussian function to estimate
     * probability of a given class.
     *
     * @example
     * import { GaussianNB } from 'kalimdor/naive_bayes';
     *
     * const nb = new GaussianNB();
     * const X = [[1, 20], [2, 21], [3, 22], [4, 22]];
     * const y = [1, 0, 1, 0];
     * nb.fit({ X, y });
     * nb.predict({ X: [[1, 20]] }); // returns [ 1 ]
     *
     */
    export class GaussianNB {
        /**
         * Naive Bayes summary according to classes
         */
        private summaries;
        /**
         * To clone input values
         */
        private clone;
        /**
         * @param clone - To clone the input values during fit and predict
         */
        constructor({ clone }?: {
            clone: boolean;
        });
        /**
         * Fit date to build Gaussian Distribution summary
         * @param {any} X - training values
         * @param {any} y - target values
         */
        fit({ X, y }?: {
            X: any[][];
            y: any[];
        }): void;
        /**
         * Predict multiple rows
         * @param {any[]} X - values to predict in Matrix format
         * @returns {number[]}
         */
        predict({ X }?: {
            X: any[][];
        }): number[];
        /**
         * Restores GaussianNB model from a checkpoint
         * @param summaries - Gaussian Distribution summaries
         */
        fromJSON({ summaries }?: {
            summaries: {};
        }): void;
        /**
         * Returns a model checkpoint
         */
        toJSON(): {
            summaries: {};
        };
        /**
         * Make a prediction
         * @param X -
         */
        private singlePredict;
        /**
         * Calculate the main division
         * @param x
         * @param meanval
         * @param stdev
         */
        private calculateProbability;
        /**
         * Summarise the dataset per class using "probability density function"
         * example:
         * Given
         * const X = [[1,20], [2,21], [3,22], [4,22]];
         * const y = [1, 0, 1, 0];
         * Returns
         * { '0': [ [ 3, 1.4142135623730951 ], [ 21.5, 0.7071067811865476 ] ],
         * '1': [ [ 2, 1.4142135623730951 ], [ 21, 1.4142135623730951 ] ] }
         * @param dataset
         */
        private summarizeByClass;
        /**
         * Summarise the dataset to calculate the ‘pdf’ (probability density function) later on
         * @param dataset
         */
        private summarize;
        /**
         * Separates X by classes specified by y argument
         * @param X
         * @param y
         */
        private separateByClass;
    }
}
declare module "lib/naive_bayes/index.repl" { }
declare module "lib/naive_bayes/index" {
    import { GaussianNB } from "lib/naive_bayes/gaussian";
    export { GaussianNB };
}
declare module "lib/neighbors/index.repl" { }
declare module "lib/preprocessing/index.repl" { }
declare module "lib/svm/index.repl" { }
declare module "lib/tree/index.repl" { }
