/**
 * Ordinary base class for SGD classier or regressor
 * @ignore
 */
declare class BaseSGD {
    protected scaler: any;
    private preprocess;
    private learningRate;
    private epochs;
    private coefficients;
    private clone;
    /**
     * @param preprocess - preprocess methodology can be either minmax or null. Default is minmax.
     * @param learning_rate - Used to limit the amount each coefficient is corrected each time it is updated.
     * @param epochs - Number of iterations.
     * @param clone - To clone the passed in dataset.
     */
    constructor({ preprocess, learning_rate, epochs, clone }?: {
        preprocess?: string;
        learning_rate?: number;
        epochs?: number;
        clone?: boolean;
    });
    /**
     * Train the base SGD
     * @param X - Matrix of data
     * @param y - Matrix of targets
     */
    fit({ X, y }?: {
        X: number[][];
        y: number[];
    }): void;
    /**
     * Save the model's checkpoint
     */
    toJSON(): {
        /**
         * coefficient values
         */
        coefficients: number[];
        /**
         * scaler object such as MinMaxScaler
         */
        scaler: {};
    };
    /**
     * Restore the model from a checkpoint
     * @param coefficients - coefficient values
     * @param scaler - scaler object such as MinMaxScaler
     */
    fromJSON({ coefficients, scaler }?: {
        coefficients: number[];
        scaler: {};
    }): void;
    /**
     * Predictions according to the passed in test set
     * @param X - Matrix of data
     */
    protected predict({ X }: {
        X: any;
    }): number[];
    /**
     * Calculate yhat of a row in the dataset
     * @param row
     * @param coefficients
     */
    private predictOne;
    /**
     * SGD based on linear model to calculate coefficient
     * @param X - training data
     * @param y - target data
     */
    private sgd;
}
/**
 * Linear classifiers (SVM, logistic regression, a.o.) with SGD training.
 *
 * This estimator implements regularized linear models with
 * stochastic gradient descent (SGD) learning: the gradient of
 * the loss is estimated each sample at a time and the model is
 * updated along the way with a decreasing strength schedule
 * (aka learning rate). SGD allows minibatch (online/out-of-core)
 * learning, see the partial_fit method. For best results using
 * the default learning rate schedule, the data should have zero mean
 * and unit variance.
 *
 * @example
 * import { SGDClassifier } from 'kalimdor/linear_model';
 * const clf = new SGDClassifier();
 * const X = [[0., 0.], [1., 1.]];
 * const y = [0, 1];
 * clf.fit({ X, y });
 * clf.predict({ X: [[2., 2.]] }); // result: [ 1 ]
 *
 */
export declare class SGDClassifier extends BaseSGD {
    /**
     * Predicted values with Math.round applied
     * @param X - Matrix of data
     */
    predict({ X }?: {
        X: number[][];
    }): number[];
}
/**
 * Linear model fitted by minimizing a regularized empirical loss with SGD
 * SGD stands for Stochastic Gradient Descent: the gradient of the loss
 * is estimated each sample at a time and the model is updated along
 * the way with a decreasing strength schedule (aka learning rate).
 *
 * @example
 * import { SGDRegressor } from 'kalimdor/linear_model';
 * const reg = new SGDRegressor();
 * const X = [[0., 0.], [1., 1.]];
 * const y = [0, 1];
 * reg.fit({ X, y });
 * reg.predict({ X: [[2., 2.]] }); // result: [ 1.281828588248001 ]
 *
 */
export declare class SGDRegressor extends BaseSGD {
    /**
     * Predicted values
     * @param X - Matrix of data
     */
    predict({ X }?: {
        X: number[][];
    }): number[];
}
export {};
