export declare type Type = 'C_SVC' | 'NU_SVC' | 'ONE_CLASS' | 'EPSILON_SVR' | 'NU_SVR';
export declare type Kernel = 'LINEAR' | 'POLYNOMIAL' | 'RBF' | 'SIGMOID';
/**
 * Options used by sub classes
 * Notice type is disabled as they are set statically from children classes
 */
export interface SVMOptions {
    /**
     * Type of SVM
     */
    type?: Type;
    /**
     * Degree of polynomial, test for polynomial kernel
     */
    degree?: number;
    /**
     * Type of Kernel
     */
    kernel?: Kernel;
    /**
     * Gamma parameter of the RBF, Polynomial and Sigmoid kernels. Default value is 1/num_features
     */
    gamma?: number | null;
    /**
     * coef0 parameter for Polynomial and Sigmoid kernels
     */
    coef0?: number;
    /**
     * Cost parameter, for C SVC, Epsilon SVR and NU SVR
     */
    cost?: number;
    /**
     * For NU SVC and NU SVR
     */
    nu?: number;
    /**
     * For epsilon SVR
     */
    epsilon?: number;
    /**
     * Cache size in MB
     */
    cacheSize?: number;
    /**
     * Tolerance
     */
    tolerance?: number;
    /**
     * Use shrinking euristics (faster)
     */
    shrinking?: boolean;
    /**
     * weather to train SVC/SVR model for probability estimates,
     */
    probabilityEstimates?: boolean;
    /**
     * Set weight for each possible class
     */
    weight?: object | null;
    /**
     * Print info during training if false (aka verbose)
     */
    quiet?: boolean;
}
/**
 * BaseSVM class used by all parent SVM classes that are based on libsvm
 */
export declare class BaseSVM {
    static create(options?: SVMOptions): Promise<BaseSVM>;
    /**
     * Get Kernel name type using string Kernel name
     * @param SVM
     * @param {string} name
     * @returns {number}
     */
    private static getKernel;
    /**
     * Get Kernel type using string type name
     * @param SVM
     * @param {string} name
     * @returns {number}
     */
    private static getType;
    /**
     * Get a consolidated options including type and Kernel
     * @param SVM
     * @param {Options} options
     * @param {Type} type
     * @param {Kernel} kernel
     * @returns {Object}
     */
    private static processOptions;
    protected svm: any;
    protected options: SVMOptions;
    protected constructor(svm: any, options: SVMOptions);
    /**
     * Fit the model according to the given training data.
     * @param {number[][]} X
     * @param {number[]} y
     * @returns {Promise<void>}
     */
    fit({ X, y }: {
        X: number[][];
        y: number[];
    }): Promise<void>;
    /**
     * Predict using the linear model
     * @param {number[][]} X
     * @returns {number[]}
     */
    predict(X: number[][]): number[];
    /**
     * Predict the label of one sample.
     * @param {number[]} X
     * @returns {number}
     */
    predictOne(X: number[]): number;
    /**
     * Saves the current SVM as a JSON object
     * @returns {{svm: string; options: SVMOptions}}
     */
    toJSON(): {
        svm: any;
        options: SVMOptions;
    };
    /**
     * Restores the model from a JSON checkpoint
     * @param {any} svm
     * @param {any} type
     * @param {any} options
     */
    fromJSON(serialized: any): Promise<void>;
}
/**
 * C-Support Vector Classification.
 *
 * The implementation is based on libsvm. The fit time complexity is more than
 * quadratic with the number of samples which makes it hard to scale to dataset
 * with more than a couple of 10000 samples.
 *
 * The multiclass support is handled according to a one-vs-one scheme.
 *
 * For details on the precise mathematical formulation of the provided kernel
 * functions and how gamma, coef0 and degree affect each other, see the corresponding
 * section in the narrative documentation: Kernel functions.
 */
export declare class SVC extends BaseSVM {
    static create(options?: SVMOptions): Promise<SVC>;
}
/**
 * Linear Support Vector Regression.
 *
 * Similar to SVR with parameter kernel=’linear’, but implemented in terms of
 * liblinear rather than libsvm, so it has more flexibility in the choice of
 * penalties and loss functions and should scale better to large numbers of samples.
 *
 * This class supports both dense and sparse input.
 */
export declare class SVR extends BaseSVM {
    static create(options?: SVMOptions): Promise<SVR>;
}
/**
 * Unsupervised Outlier Detection.
 *
 * Estimate the support of a high-dimensional distribution.
 *
 * The implementation is based on libsvm.
 */
export declare class OneClassSVM extends BaseSVM {
    static create(options?: SVMOptions): Promise<OneClassSVM>;
}
/**
 * Nu-Support Vector Classification.
 *
 * Similar to SVC but uses a parameter to control the number of support vectors.
 *
 * The implementation is based on libsvm.
 */
export declare class NuSVC extends BaseSVM {
    static create(options?: SVMOptions): Promise<NuSVC>;
}
/**
 * Nu Support Vector Regression.
 *
 * Similar to NuSVC, for regression, uses a parameter nu to control the number
 * of support vectors. However, unlike NuSVC, where nu replaces C, here nu
 * replaces the parameter epsilon of epsilon-SVR.
 *
 * The implementation is based on libsvm.
 */
export declare class NuSVR extends BaseSVM {
    static create(options?: SVMOptions): Promise<NuSVR>;
}
