/**
 * Question used by decision tree algorithm to determine whether to split branch or not
 * @ignore
 */
export declare class Question {
    private features;
    private column;
    private value;
    constructor(features: any, column: any, value: any);
    match(example: any): boolean;
    toString(): string;
}
/**
 * According to the given targets array, count occurrences into an object.
 * @param {any[]} targets - list of class: count
 * @returns {}
 * @ignore
 */
export declare function classCounts(targets: any[]): {};
/**
 * A leaf node that classifies data.
 * @ignore
 */
export declare class Leaf {
    prediction: any;
    constructor(y: any);
}
/**
 * It holds a reference to the question, and to the two children nodes
 * @ignore
 */
export declare class DecisionNode {
    question: any;
    trueBranch: any;
    falseBranch: any;
    constructor(question: any, trueBranch: any, falseBranch: any);
}
/**
 * A decision tree classifier.
 *
 * @example
 * import { DecisionTreeClassifier } from 'kalimdor/tree';
 * const features = ['color', 'diameter', 'label'];
 * const decision = new DecisionTreeClassifier({ featureLabels: features });
 *
 * const X = [['Green', 3], ['Yellow', 3], ['Red', 1], ['Red', 1], ['Yellow', 3]];
 * const y = ['Apple', 'Apple', 'Grape', 'Grape', 'Lemon'];
 * decision.fit({ X, y });
 * decision.printTree(); // try it out yourself! =)
 *
 * decision.predict({ X: [['Green', 3]] }); // [ 'Apple' ]
 * decision.predict({ X }); // [ [ 'Apple' ], [ 'Apple', 'Lemon' ], [ 'Grape', 'Grape' ], [ 'Grape', 'Grape' ], [ 'Apple', 'Lemon' ] ]
 *
 * @example
 * import { DecisionTreeClassifier } from 'kalimdor/tree';
 * const decision = new DecisionTreeClassifier({ featureLabels: null });
 *
 * const X = [[0, 0], [1, 1]];
 * const Y = [0, 1];
 * decision.fit({ X, y });
 * decision2.predict({ row: [[2, 2]] }); // [ 1 ]
 */
export interface Options {
    featureLabels?: null | any[];
    verbose?: boolean;
    randomise?: boolean;
}
export declare class DecisionTreeClassifier {
    private featureLabels;
    private tree;
    private verbose;
    private randomise;
    private randomEngine;
    constructor({ featureLabels, verbose, randomise, random_state, tree }?: {
        featureLabels?: any[];
        verbose?: boolean;
        randomise?: boolean;
        random_state?: number;
        tree?: any;
    });
    /**
     * Fit date, which builds a tree
     * @param {any} X
     * @param {any} y
     * @returns {Leaf | DecisionNode}
     */
    fit({ X, y }: {
        X: any[];
        y: any[];
    }): void;
    /**
     * Predict multiple rows
     * @param {any[]} X
     * @returns {any[]}
     */
    predict({ X }: {
        X: any[][];
    }): any;
    /**
     * Returns the model checkpoint
     * @returns {{featureLabels: string[]; tree: any; verbose: boolean; randomise: boolean}}
     */
    toJSON(): {
        featureLabels: string[];
        tree: any;
        verbose: boolean;
        randomise: boolean;
    };
    /**
     * Restores the model from a checkpoint
     * @param {string[]} featureLabels
     * @param {any} tree
     * @param {boolean} verbose
     * @param {boolean} randomise
     */
    fromJSON({ featureLabels, tree, verbose, randomise }: {
        featureLabels: string[];
        tree: any;
        verbose: boolean;
        randomise: boolean;
    }): void;
    /**
     * Recursively print the tree into console
     * @param {string} spacing
     */
    printTree(spacing?: string): void;
    /**
     * Partition X and y into true and false branches
     * @param X
     * @param y
     * @param {Question} question
     * @returns {{trueX: Array<any>; trueY: Array<any>; falseX: Array<any>; falseY: Array<any>}}
     */
    private partition;
    /**
     * Calculate the gini impurity of rows
     * Checkout: https://en.wikipedia.org/wiki/Decision_tree_learning#Gini_impurity
     * @param targets
     * @returns {number}
     */
    private gini;
    /**
     * Information Gain.
     *
     * The uncertainty of the starting node, minus the weighted impurity of
     * two child nodes.
     * @param left
     * @param right
     * @param uncertainty
     * @returns {number}
     */
    private infoGain;
    /**
     * Find the best split for the current X and y.
     * @param X
     * @param y
     * @param {boolean} randomise
     * @returns {{bestGain: number; bestQuestion: any}}
     */
    private findBestSplit;
    /**
     * Interactively build tree until it reaches the terminal nodes
     * @param {any} X
     * @param {any} y
     * @returns {any}
     */
    private buildTree;
    /**
     * Internal predict method separated out for recursion purpose
     * @param {any} row
     * @param {any} node
     * @returns {any}
     * @private
     */
    private _predict;
    /**
     * Private method for printing tree; required for recursion
     * @param {any} node
     * @param {any} spacing
     */
    private _printTree;
}
