"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
var fs = require("fs");
var _ = require("lodash");
var path = require("path");
var BaseProcesser_1 = require("./BaseProcesser");
var consts = require("./const");
var docsJson = JSON.parse(fs.readFileSync(path.join(__dirname, '../docs.json'), 'utf8'));
/**
 * Processor used to process API docs located under lib/src/
 */
var APIProcessor = /** @class */ (function (_super) {
    __extends(APIProcessor, _super);
    function APIProcessor() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.apiChildren = [];
        _this.vuepressExtraConfigPath = path.join(__dirname, '../md_out/.vuepress/apiExtra.json');
        _this.themePath = path.join(__dirname, '../themes/markdown');
        _this.entityPageFile = 'entity_page.hbs';
        _this.homePageFile = 'api_readme.hbs';
        _this.apiOutputPath = path.join(__dirname, '../md_out/api');
        _this.srcApiHomeTheme = path.join(_this.themePath, _this.homePageFile);
        _this.destApiHomePage = path.join(__dirname, '../md_out/api/README.md');
        _this.pathDelimeter = '.';
        _this.entityKindWhitelist = [consts.kindStringClass, consts.kindStringFunction]; // Whitelisting kinds when grabbing class or method
        _this.moduleNameBlackList = ['"'];
        /**
         * Util funciton to clean any unwanted chars
         * @param name
         * @returns {string}
         */
        _this.cleanName = function (name) {
            return _.reduce(_this.moduleNameBlackList, function (filteredName, blackKey) {
                return _.replace(filteredName, blackKey, '');
            }, name);
        };
        return _this;
    }
    /**
     * Run the processor
     * @param hbs
     */
    APIProcessor.prototype.run = function (hbs) {
        // Creating required dir
        this.createDir();
        // Order API children
        this.apiChildren = this.retrieveOrderedAPIs(docsJson);
        // Construct the sidebar configs
        this.buildSidebar(this.apiChildren);
        // Create the API homepage
        this.processHomePage(hbs, this.apiChildren);
        // Process API pages
        this.processAPIEntityPage(hbs, this.apiChildren);
    };
    /**
     * Build the sidebar json for the API pages
     * @param apiChildren
     */
    APIProcessor.prototype.buildSidebar = function (apiChildren) {
        var extraConfig = {
            apiSidebar: this.buildSidebarJSON(apiChildren)
        };
        // Writing extraConfig object as .vuepress/apiExtra.json
        fs.writeFileSync(this.vuepressExtraConfigPath, JSON.stringify(extraConfig), 'utf-8');
    };
    /**
     * Build a sidebar JSON for nested navigations
     * e.g. [{"title":"cluster","collapsable":false,"children":[["./cluster.KMeans","KMeans"]]}
     * @param apiChildren
     * @returns {Array}
     */
    APIProcessor.prototype.buildSidebarJSON = function (apiChildren) {
        return _.reduce(apiChildren, function (sum, child) {
            var _a = child.name.split('.'), module = _a[0], name = _a[1];
            var existingGroupIndex = _.findIndex(sum, function (o) { return o.title === module; });
            if (existingGroupIndex === -1) {
                // If there's no existing module group according to the current child's name
                // create a new definition and append it to the sum
                var newDefinition = {
                    children: [["./" + child.name, name]],
                    collapsable: false,
                    title: module
                };
                return _.concat(sum, [newDefinition]);
            }
            else {
                // If there's an existing module definition,
                // then append the current child's definition to the children list
                var existing = sum[existingGroupIndex];
                var newChildren = _.concat(existing.children, [["./" + child.name, name]]);
                var updated = _.set(existing, 'children', newChildren);
                return _.set(sum, "[" + existingGroupIndex + "]", updated);
            }
        }, []);
    };
    /**
     * Retrieve ordered APIs using raw docs generated from typedoc --json
     * @param docs
     * @returns {any[]}
     */
    APIProcessor.prototype.retrieveOrderedAPIs = function (docs) {
        var _this = this;
        // Aggregate children
        var aggregatedFirstChildren = _.reduce(docs.children, function (aggregation, moduleChild) {
            // Looping the first children layer
            // Group child according the module name
            var _a = moduleChild.name.split('/'), module = _a[0], file = _a[1];
            // Clean any unwanted chars from the modulen name
            var cleanedModuleName = _this.cleanName(module);
            // Grabbing each class or method of the module
            // Also it squashes the entities by moduelName.entityName e.g. preprocessing.OneHotEncoder
            var squashedEntityList = _.reduce(moduleChild.children, function (entityList, entityChild) {
                // Filter by entityKindWhitelist and skips if isIgnore comment is set
                if (_this.entityKindWhitelist.indexOf(entityChild.kindString) !== -1 && !_this.isIgnore(entityChild)) {
                    // each function or class name
                    var entityName = entityChild.name;
                    var fullEntityName = [cleanedModuleName, entityName].join(_this.pathDelimeter);
                    var newEntityChild = _.set(entityChild, 'name', fullEntityName);
                    return _.concat(entityList, [newEntityChild]);
                }
                return entityList;
            }, []);
            // Filter out undefined entity appended as a result of whitelisting during the reduce
            var filteredEntityList = _.filter(squashedEntityList, function (x) { return !_.isUndefined(x); });
            // Concat the squashedEntityList to the _.reduce aggregation
            // Also applies a shallow flatten as squashedEntityList is an array
            return _.flatten(_.concat(aggregation, filteredEntityList));
        }, []);
        // Ordering each entity by its name
        return _.orderBy(aggregatedFirstChildren, ['name']);
    };
    /**
     * Create API directory if not exist
     */
    APIProcessor.prototype.createDir = function () {
        // 1.2. creating the second portion: /Users/jasons/Desktop/kalimdorjs/docs/md_out/pages
        if (!fs.existsSync(this.apiOutputPath)) {
            fs.mkdirSync(this.apiOutputPath);
        }
    };
    /**
     * Process API folder's homepage aka README
     */
    APIProcessor.prototype.processHomePage = function (hbs, apiChildren) {
        var _this = this;
        var grouped = _.groupBy(apiChildren, function (o) { return o.name.split(_this.pathDelimeter)[0]; });
        var keys = _.keys(grouped);
        var restructedChildren = _.map(keys, function (key) {
            return {
                key: key,
                value: _.get(grouped, key)
            };
        });
        var apiHomePageThemeContent = fs.readFileSync(this.srcApiHomeTheme, 'utf8');
        var template = hbs.compile(apiHomePageThemeContent);
        var compiledPage = template(restructedChildren);
        fs.appendFileSync(this.destApiHomePage, compiledPage, { flag: 'a' });
    };
    /**
     * Check if the passed in child has a commnet "ignore"
     * 1. If child does not have comment, search for an identical signature and pull out @ignore tag if it exists
     * @param child
     */
    APIProcessor.prototype.isIgnore = function (child) {
        // Find ignores from given tags
        var findIgnores = function (givenTags) {
            return _.find(givenTags, function (tag) {
                return tag.tag === consts.tagTypeIgnore;
            });
        };
        // If child does not have comment attribute by default, then search for a signature
        if (!child.comment) {
            var signatures = child.signatures;
            var identSignature = _.find(signatures, function (sig) {
                return sig.name === child.name;
            });
            var foundTags = _.get(identSignature, 'comment.tags', []);
            var foundIgnores = findIgnores(foundTags);
            return !_.isEmpty(foundIgnores);
        }
        // Otherwise, evaluate using the parent comment
        var tags = _.get(child, 'comment.tags', []);
        var ignore = findIgnores(tags);
        return !_.isEmpty(ignore);
    };
    /**
     * Processes API entity pages
     * @param hbs
     * @param children
     */
    APIProcessor.prototype.processAPIEntityPage = function (hbs, children) {
        var _this = this;
        // themes hbs files paths
        var entityPageThemePath = path.join(this.themePath, this.entityPageFile);
        var entityPageThemeContent = fs.readFileSync(entityPageThemePath, 'utf8');
        _.forEach(children, function (entityChild) {
            // 1. pages/
            // - create pages using the content
            var fullPath = path.join(_this.apiOutputPath, entityChild.name + ".md");
            var template = hbs.compile(entityPageThemeContent);
            var compiledPage = template(entityChild);
            fs.appendFileSync(fullPath, compiledPage, { flag: 'a' });
        });
    };
    return APIProcessor;
}(BaseProcesser_1.BaseProcesser));
exports.APIProcessor = APIProcessor;
